@@ -7,8 +7,9 @@
  * Copyright (C) 1998-2001 Michael Meeks (miguel@kernel.org)
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) version 3.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -142,7 +143,7 @@ excel_read_metadata (GsfDocMetaData *meta_data, GsfInfile *ole, char const *name
 static void
 cb_dump_vba (char const *name, guint8 const *src_code)
 {
-	printf ("<module name=\"%s\">\n<![CDATA[%s]]>\n</module>\n", name, src_code);
+	g_printerr ("<module name=\"%s\">\n<![CDATA[%s]]>\n</module>\n", name, src_code);
 }
 #endif
 
@@ -346,18 +347,18 @@ go_plugin_init (GOPlugin *plugin, GOCmdContext *cc)
 			continue;
 		if (fd->flags & XL_XLM) {
 			if (fd->flags != XL_XLM)
-				fprintf (stderr, "%s : flags in addition to XLM\n", name);
+				g_printerr ("%s : flags in addition to XLM\n", name);
 			if (fd->min_args != fd->max_args)
-				fprintf (stderr, "%s : min != max\n", name);
+				g_printerr ("%s : min != max\n", name);
 			continue;
 		}
 		if (fd->min_args < 0)
-			fprintf (stderr, "%s : min_args < 0\n", name);
+			g_printerr ("%s : min_args < 0\n", name);
 		if (fd->max_args < 0)
-			fprintf (stderr, "%s : min_args < 0\n", name);
+			g_printerr ("%s : min_args < 0\n", name);
 		if (fd->known_args != NULL &&
 		    fd->num_known_args != strlen (fd->known_args))
-			fprintf (stderr, "%s : num_expected_args inconsistent\n", name);
+			g_printerr ("%s : num_expected_args inconsistent\n", name);
 	}
 }
 #endif
