@@ -1004,3 +1004,19 @@ gnm_cell_convert_expr_to_value (GnmCell *cell)
 	cell->base.texpr = NULL;
 }
 
+static gpointer cell_boxed_copy (gpointer c) { return c; }
+static void cell_boxed_free (gpointer c) { }
+
+GType
+gnm_cell_get_type (void)
+{
+    static GType type_cell = 0;
+
+    if (!type_cell)
+	type_cell = g_boxed_type_register_static
+	    ("GnmCell",
+	     cell_boxed_copy,
+	     cell_boxed_free);
+
+    return type_cell;
+}
