@@ -285,78 +285,80 @@ cb_c_fmt_dialog_edit_style_button (G_GNUC_UNUSED GtkWidget *btn, CFormatState *s
 static GnmStyleCond *
 c_fmt_dialog_get_condition (CFormatState *state)
 {
-	GnmStyleCond *cond = g_new0(GnmStyleCond, 1);
+	GnmStyleCondOp op;
+	GnmStyleCond *cond;
 	GtkTreeIter iter;
 	gint n_expr = 0;
 	GnmParsePos pp;
+	GnmStyle *overlay;
 
 	parse_pos_init_editpos (&pp, state->sv);
 
-	cond->overlay = gnm_style_new ();
+	overlay = gnm_style_new ();
 	if (state->editor.style) {
 		if (cb_c_fmt_dialog_chooser_check_page
 		    (state, "check-background", FD_BACKGROUND)) {
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_COLOR_BACK);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_COLOR_PATTERN);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_PATTERN);
 		}
 		if (cb_c_fmt_dialog_chooser_check_page
 		    (state, "check-number", FD_NUMBER)) {
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_FORMAT);
 		}
 		if (cb_c_fmt_dialog_chooser_check_page
 		    (state, "check-align", FD_ALIGNMENT)) {
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_ALIGN_V);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_ALIGN_H);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_INDENT);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_ROTATION);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_TEXT_DIR);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_WRAP_TEXT);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_SHRINK_TO_FIT);
 		}
 		if (cb_c_fmt_dialog_chooser_check_page
 		    (state, "check-font", FD_FONT)) {
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_FONT_COLOR);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_FONT_NAME);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_FONT_BOLD);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_FONT_ITALIC);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_FONT_UNDERLINE);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_FONT_STRIKETHROUGH);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_FONT_SCRIPT);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_FONT_SIZE);
 		}
 		if (cb_c_fmt_dialog_chooser_check_page
 		    (state, "check-border", FD_BORDER)) {
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_BORDER_TOP);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_BORDER_BOTTOM);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_BORDER_LEFT);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_BORDER_RIGHT);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_BORDER_REV_DIAGONAL);
-			gnm_style_merge_element (cond->overlay, state->editor.style,
+			gnm_style_merge_element (overlay, state->editor.style,
 						 MSTYLE_BORDER_DIAGONAL);
 		}
 		if (cb_c_fmt_dialog_chooser_check_page
@@ -371,23 +373,29 @@ c_fmt_dialog_get_condition (CFormatState *state)
 	if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (state->editor.combo), &iter))
 		gtk_tree_model_get (GTK_TREE_MODEL (state->editor.typestore),
 				    &iter,
-				    1, &cond->op,
+				    1, &op,
 				    2, &n_expr,
 				    -1);
 	else
-		cond->op = GNM_STYLE_COND_CONTAINS_ERR;
+		op = GNM_STYLE_COND_CONTAINS_ERR;
+
+	cond = gnm_style_cond_new (op, state->sheet);
+	gnm_style_cond_set_overlay (cond, overlay);
+	gnm_style_unref (overlay);
 
 	if (n_expr > 0) {
 		GnmExprTop const *texpr = gnm_expr_entry_parse (GNM_EXPR_ENTRY (state->editor.expr_x), &pp,
 								NULL, FALSE,
 								GNM_EXPR_PARSE_UNKNOWN_NAMES_ARE_STRINGS);
-		cond->texpr[0] = texpr;
+		gnm_style_cond_set_expr (cond, texpr, 0);
+		gnm_expr_top_unref (texpr);
 	}
 	if (n_expr > 1) {
 		GnmExprTop const *texpr = gnm_expr_entry_parse (GNM_EXPR_ENTRY (state->editor.expr_y), &pp,
 								NULL, FALSE,
 								GNM_EXPR_PARSE_UNKNOWN_NAMES_ARE_STRINGS);
-		cond->texpr[1] = texpr;
+		gnm_style_cond_set_expr (cond, texpr, 1);
+		gnm_expr_top_unref (texpr);
 	}
 	return cond;
 }
@@ -397,7 +405,7 @@ cb_c_fmt_dialog_add_button (G_GNUC_UNUSED GtkWidget *btn, CFormatState *state)
 {
 	GnmStyleCond *cond = c_fmt_dialog_get_condition (state);
 	c_fmt_dialog_apply_add_choice (state, cond, TRUE);
-	g_free (cond);
+	gnm_style_cond_free (cond);
 }
 
 static void
@@ -405,7 +413,7 @@ cb_c_fmt_dialog_replace_button (G_GNUC_UNUSED GtkWidget *btn, CFormatState *stat
 {
 	GnmStyleCond *cond = c_fmt_dialog_get_condition (state);
 	c_fmt_dialog_apply_add_choice (state, cond, FALSE);
-	g_free (cond);
+	gnm_style_cond_free (cond);
 }
 
 static void
@@ -418,10 +426,10 @@ cb_c_fmt_dialog_copy_button (G_GNUC_UNUSED GtkWidget *btn, CFormatState *state)
 		GtkTreePath *path = gtk_tree_model_get_path
 			(GTK_TREE_MODEL (state->model), &iter);
 		gint *pind = gtk_tree_path_get_indices (path);
-		GArray const *conds = gnm_style_conditions_details (sc);
+		GPtrArray const *conds = gnm_style_conditions_details (sc);
 		if (pind && conds) {
 			gint ind = *pind;
-			GnmStyleCond *gsc = &g_array_index (conds, GnmStyleCond, ind);
+			GnmStyleCond *gsc = g_ptr_array_index (conds, ind);
 			GtkTreeIter iter;
 			GnmParsePos pp;
 			GnmStyle   *style;
@@ -444,16 +452,16 @@ cb_c_fmt_dialog_copy_button (G_GNUC_UNUSED GtkWidget *btn, CFormatState *state)
 			}
 			/* Set the expressions */
 			parse_pos_init_editpos (&pp, state->sv);
-			if (gsc->texpr[0])
+			if (gnm_style_cond_get_expr (gsc, 0))
 				gnm_expr_entry_load_from_expr (GNM_EXPR_ENTRY (state->editor.expr_x),
-							       gsc->texpr[0],
+							       gnm_style_cond_get_expr (gsc, 0),
 							       &pp);
 			else
 				gnm_expr_entry_load_from_text (GNM_EXPR_ENTRY (state->editor.expr_x),
 							       "");
-			if (gsc->texpr[1])
+			if (gnm_style_cond_get_expr (gsc, 1))
 				gnm_expr_entry_load_from_expr (GNM_EXPR_ENTRY (state->editor.expr_y),
-							       gsc->texpr[1],
+							       gnm_style_cond_get_expr (gsc, 1),
 							       &pp);
 			else
 				gnm_expr_entry_load_from_text (GNM_EXPR_ENTRY (state->editor.expr_y),
@@ -671,7 +679,7 @@ c_fmt_dialog_apply_add_choice (CFormatState *state, GnmStyleCond *cond, gboolean
 		int index = -1;
 		sc = gnm_style_conditions_dup (gnm_style_get_conditions (state->style));
 		if (sc == NULL)
-			sc = gnm_style_conditions_new ();
+			sc = gnm_style_conditions_new (state->sheet);
 		if (!add) {
 			GtkTreeIter iter;
 			if (gtk_tree_selection_get_selected (state->selection, NULL, &iter)) {
@@ -803,13 +811,13 @@ static void
 c_fmt_dialog_conditions_page_load_cond_double_f (CFormatState *state,
 					       GnmStyleCond const *cond, GtkTreeIter *iter1)
 {
-	c_fmt_dialog_conditions_page_load_cond_single_f (state, cond->texpr[0], iter1);
-	c_fmt_dialog_conditions_page_load_cond_single_f (state, cond->texpr[1], iter1);
+	c_fmt_dialog_conditions_page_load_cond_single_f (state, gnm_style_cond_get_expr (cond, 0), iter1);
+	c_fmt_dialog_conditions_page_load_cond_single_f (state, gnm_style_cond_get_expr (cond, 1), iter1);
 }
 
 static void
 c_fmt_dialog_conditions_page_load_cond (CFormatState *state, GnmStyleCond const *cond,
-				      GtkTreeIter *iter)
+					GtkTreeIter *iter)
 {
 	GtkTreeIter iter1;
 
@@ -838,7 +846,7 @@ c_fmt_dialog_conditions_page_load_cond (CFormatState *state, GnmStyleCond const
 				    _("If the cell content is equal to this value"
 				      ", a special style is used."),
 				    CONDITIONS_REFERENCE, NULL, -1);
-		c_fmt_dialog_conditions_page_load_cond_single_f (state, cond->texpr[0], &iter1);
+		c_fmt_dialog_conditions_page_load_cond_single_f (state, gnm_style_cond_get_expr (cond, 0), &iter1);
 		break;
 	case GNM_STYLE_COND_NOT_EQUAL:
 		gtk_tree_store_set (state->model, &iter1, CONDITIONS_RANGE, NULL,
@@ -846,14 +854,14 @@ c_fmt_dialog_conditions_page_load_cond (CFormatState *state, GnmStyleCond const
 				    _("If the cell content is not equal to this value"
 				      ", a special style is used."),
 				    CONDITIONS_REFERENCE, NULL, -1);
-		c_fmt_dialog_conditions_page_load_cond_single_f (state, cond->texpr[0], &iter1);
+		c_fmt_dialog_conditions_page_load_cond_single_f (state, gnm_style_cond_get_expr (cond, 0), &iter1);
 		break;
 	case GNM_STYLE_COND_GT:
 		gtk_tree_store_set (state->model, &iter1, CONDITIONS_RANGE, NULL,
 				    CONDITIONS_COND,
 				    _("If the cell content is > this value, a "
 				      "special style is used."), -1);
-		c_fmt_dialog_conditions_page_load_cond_single_f (state, cond->texpr[0], &iter1);
+		c_fmt_dialog_conditions_page_load_cond_single_f (state, gnm_style_cond_get_expr (cond, 0), &iter1);
 		break;
 	case GNM_STYLE_COND_LT:
 		gtk_tree_store_set (state->model, &iter1, CONDITIONS_RANGE, NULL,
@@ -861,7 +869,7 @@ c_fmt_dialog_conditions_page_load_cond (CFormatState *state, GnmStyleCond const
 				    _("If the cell content is < this value, a "
 				      "special style is used."),
 				    CONDITIONS_REFERENCE, NULL, -1);
-		c_fmt_dialog_conditions_page_load_cond_single_f (state, cond->texpr[0], &iter1);
+		c_fmt_dialog_conditions_page_load_cond_single_f (state, gnm_style_cond_get_expr (cond, 0), &iter1);
 		break;
 	case GNM_STYLE_COND_GTE:
 		gtk_tree_store_set (state->model, &iter1, CONDITIONS_RANGE, NULL,
@@ -870,7 +878,7 @@ c_fmt_dialog_conditions_page_load_cond (CFormatState *state, GnmStyleCond const
 				      "value, a special style is used."),
 				    CONDITIONS_REFERENCE, NULL, -1);
 
-		c_fmt_dialog_conditions_page_load_cond_single_f (state, cond->texpr[0], &iter1);
+		c_fmt_dialog_conditions_page_load_cond_single_f (state, gnm_style_cond_get_expr (cond, 0), &iter1);
 		break;
 	case GNM_STYLE_COND_LTE:
 		gtk_tree_store_set (state->model, &iter1, CONDITIONS_RANGE, NULL,
@@ -878,7 +886,7 @@ c_fmt_dialog_conditions_page_load_cond (CFormatState *state, GnmStyleCond const
 				    _("If the cell content is \xe2\x89\xa6 this "
 				      "value, a special style is used."),
 				    CONDITIONS_REFERENCE, NULL, -1);
-		c_fmt_dialog_conditions_page_load_cond_single_f (state, cond->texpr[0], &iter1);
+		c_fmt_dialog_conditions_page_load_cond_single_f (state, gnm_style_cond_get_expr (cond, 0), &iter1);
 		break;
 
 	case GNM_STYLE_COND_CUSTOM:
@@ -886,7 +894,7 @@ c_fmt_dialog_conditions_page_load_cond (CFormatState *state, GnmStyleCond const
 				    CONDITIONS_COND,
 				    _("If this formula evaluates to TRUE, a special style is used."),
 				    CONDITIONS_REFERENCE, NULL, -1);
-		c_fmt_dialog_conditions_page_load_cond_single_f (state, cond->texpr[0], &iter1);
+		c_fmt_dialog_conditions_page_load_cond_single_f (state, gnm_style_cond_get_expr (cond, 0), &iter1);
 		break;
 	case GNM_STYLE_COND_CONTAINS_STR:
 		gtk_tree_store_set (state->model, &iter1, CONDITIONS_RANGE, NULL,
@@ -894,7 +902,7 @@ c_fmt_dialog_conditions_page_load_cond (CFormatState *state, GnmStyleCond const
 				    _("If the cell content contains this string"
 				      ", a special style is used."),
 				    CONDITIONS_REFERENCE, NULL, -1);
-		c_fmt_dialog_conditions_page_load_cond_single_f (state, cond->texpr[0], &iter1);
+		c_fmt_dialog_conditions_page_load_cond_single_f (state, gnm_style_cond_get_expr (cond, 0), &iter1);
 		break;
 	case GNM_STYLE_COND_NOT_CONTAINS_STR:
 		gtk_tree_store_set (state->model, &iter1, CONDITIONS_RANGE, NULL,
@@ -902,7 +910,7 @@ c_fmt_dialog_conditions_page_load_cond (CFormatState *state, GnmStyleCond const
 				    _("If the cell content does not contain this string"
 				      ", a special style is used."),
 				    CONDITIONS_REFERENCE, NULL, -1);
-		c_fmt_dialog_conditions_page_load_cond_single_f (state, cond->texpr[0], &iter1);
+		c_fmt_dialog_conditions_page_load_cond_single_f (state, gnm_style_cond_get_expr (cond, 0), &iter1);
 		break;
 	case GNM_STYLE_COND_BEGINS_WITH_STR:
 		gtk_tree_store_set (state->model, &iter1, CONDITIONS_RANGE, NULL,
@@ -910,14 +918,14 @@ c_fmt_dialog_conditions_page_load_cond (CFormatState *state, GnmStyleCond const
 				    _("If the cell content begins with this string"
 				      ", a special style is used."),
 				    CONDITIONS_REFERENCE, NULL, -1);
-		c_fmt_dialog_conditions_page_load_cond_single_f (state, cond->texpr[0], &iter1);
+		c_fmt_dialog_conditions_page_load_cond_single_f (state, gnm_style_cond_get_expr (cond, 0), &iter1);
 		break;
 	case GNM_STYLE_COND_NOT_BEGINS_WITH_STR:
 		gtk_tree_store_set (state->model, &iter1, CONDITIONS_RANGE, NULL,
 				    CONDITIONS_COND,
 				    _("If the cell content does not begin with this string,"
 				      " a special style is used."), -1);
-		c_fmt_dialog_conditions_page_load_cond_single_f (state, cond->texpr[0], &iter1);
+		c_fmt_dialog_conditions_page_load_cond_single_f (state, gnm_style_cond_get_expr (cond, 0), &iter1);
 		break;
 	case GNM_STYLE_COND_ENDS_WITH_STR:
 		gtk_tree_store_set (state->model, &iter1, CONDITIONS_RANGE, NULL,
@@ -925,7 +933,7 @@ c_fmt_dialog_conditions_page_load_cond (CFormatState *state, GnmStyleCond const
 				    _("If the cell content ends with this string"
 				      ", a special style is used."),
 				    CONDITIONS_REFERENCE, NULL, -1);
-		c_fmt_dialog_conditions_page_load_cond_single_f (state, cond->texpr[0], &iter1);
+		c_fmt_dialog_conditions_page_load_cond_single_f (state, gnm_style_cond_get_expr (cond, 0), &iter1);
 		break;
 	case GNM_STYLE_COND_NOT_ENDS_WITH_STR:
 		gtk_tree_store_set (state->model, &iter1, CONDITIONS_RANGE, NULL,
@@ -933,7 +941,7 @@ c_fmt_dialog_conditions_page_load_cond (CFormatState *state, GnmStyleCond const
 				    _("If the cell content does not end  "
 				      "with this string, a special style is used."),
 				    CONDITIONS_REFERENCE, NULL, -1);
-		c_fmt_dialog_conditions_page_load_cond_single_f (state, cond->texpr[0], &iter1);
+		c_fmt_dialog_conditions_page_load_cond_single_f (state, gnm_style_cond_get_expr (cond, 0), &iter1);
 		break;
 	case GNM_STYLE_COND_CONTAINS_ERR:
 		gtk_tree_store_set (state->model, &iter1, CONDITIONS_RANGE, NULL,
@@ -975,7 +983,7 @@ static void
 c_fmt_dialog_conditions_page_load_conditions (GnmStyle *style, char const *range, CFormatState *state)
 {
 	GnmStyleConditions const *sc;
-	GArray const *conds;
+	GPtrArray const *conds;
 	guint i;
 	GtkTreeIter iter1, *iter;
 
@@ -993,9 +1001,7 @@ c_fmt_dialog_conditions_page_load_conditions (GnmStyle *style, char const *range
 		}
 		for (i = 0 ; i < conds->len ; i++)
 			c_fmt_dialog_conditions_page_load_cond
-				(state, &g_array_index (conds,
-							GnmStyleCond,
-							i), iter);
+				(state, g_ptr_array_index (conds, i), iter);
 	}
 
 }
