@@ -1606,8 +1606,10 @@ validation_rebuild_validation (FormatState *state)
 			gboolean use_dropdown = gtk_toggle_button_get_active (state->validation.use_dropdown);
 			gnm_style_set_validation
 				(state->result,
-				 validation_new
-				 (style, type, op, title, msg,
+				 gnm_validation_new
+				 (style, type, op,
+				  state->sheet,
+				  title, msg,
 				  texpr0,
 				  texpr1,
 				  allow_blank,
@@ -1905,9 +1907,9 @@ fmt_dialog_init_validation_page (FormatState *state)
 		parse_pos_init (&pp, state->sheet->workbook, state->sheet,
 			state->sv->edit_pos.col, state->sv->edit_pos.row);
 		gnm_expr_entry_load_from_expr (state->validation.expr0.entry,
-			v->texpr[0], &pp);
+					       v->deps[0].texpr, &pp);
 		gnm_expr_entry_load_from_expr (state->validation.expr1.entry,
-			v->texpr[1], &pp);
+					       v->deps[1].texpr, &pp);
 	}
 
 	cb_validation_sensitivity (NULL, state);
@@ -2575,8 +2577,6 @@ dialog_cell_format_select_style (WBCGtk *wbcg, gint pages,
 
 	gtk_widget_hide (state->apply_button);
 
-	go_gtk_window_set_transient (GTK_WINDOW (w),
-				     GTK_WINDOW (state->dialog));
-	gtk_window_set_modal (GTK_WINDOW (state->dialog), TRUE);
+	go_gtk_nonmodal_dialog (w, GTK_WINDOW (state->dialog));
 	gtk_widget_show (GTK_WIDGET (state->dialog));
 }
