@@ -630,6 +630,16 @@ cb_timer_tick (SolverState *state)
 	return TRUE;
 }
 
+static void
+create_report (GnmSolver *sol, SolverState *state)
+{
+	Sheet *sheet = state->sheet;
+	char *base = g_strdup_printf ("%s Report", sheet->name_unquoted);
+	gnm_solver_create_report (sol, base);
+	g_free (base);
+}
+
+
 static GnmSolverResult *
 run_solver (SolverState *state, GnmSolverParameters *param)
 {
@@ -649,8 +659,8 @@ run_solver (SolverState *state, GnmSolverParameters *param)
 
 	sol = gnm_solver_factory_functional (param->options.algorithm,
 					     state->wbcg)
-	    ? gnm_solver_factory_create (param->options.algorithm, param)
-	    : NULL;
+		? gnm_solver_factory_create (param->options.algorithm, param)
+		: NULL;
 	if (!sol) {
 		go_gtk_notice_dialog (top, GTK_MESSAGE_ERROR,
 				      _("The chosen solver is not functional."));
@@ -801,6 +811,34 @@ run_solver (SolverState *state, GnmSolverParameters *param)
 
 		gnm_solver_store_result (sol);
 		redo = clipboard_copy_range_undo (sr.sheet, &sr.range);
+
+		if (param->options.program_report) {
+			Workbook *wb = param->sheet->workbook;
+			GOUndo *undo_report, *redo_report;
+
+			/* This is a bit of overkill -- it just removes the
+			   sheet that create_report will add.  However, if
+			   in the future we add multiple sheets then this
+			   should still be good.  */
+			undo_report = go_undo_binary_new
+				(wb,
+				 workbook_sheet_state_new (wb),
+				 (GOUndoBinaryFunc)workbook_sheet_state_restore,
+				 NULL,
+				 (GFreeFunc)workbook_sheet_state_free);
+			undo = go_undo_combine (undo, undo_report);
+
+			create_report (sol, state);
+
+			redo_report = go_undo_binary_new
+				(wb,
+				 workbook_sheet_state_new (wb),
+				 (GOUndoBinaryFunc)workbook_sheet_state_restore,
+				 NULL,
+				 (GFreeFunc)workbook_sheet_state_free);
+			redo = go_undo_combine (redo, redo_report);
+		}
+
 		cmd_generic (WORKBOOK_CONTROL (state->wbcg),
 			     _("Running solver"),
 			     undo, redo);
