@@ -148,10 +148,15 @@ clear_conditional_merges (GnmStyle *style)
 	}
 }
 
+#define MIX(H) do {				\
+  H *= G_GUINT64_CONSTANT(123456789012345);	\
+  H ^= (H >> 31);				\
+} while (0)
+
 static void
 gnm_style_update (GnmStyle *style)
 {
-	guint32 hash = 0;
+	guint64 hash = 0;
 	int i;
 
 	g_return_if_fail (style->changed);
@@ -159,103 +164,148 @@ gnm_style_update (GnmStyle *style)
 	style->changed = 0;
 
 	clear_conditional_merges (style);
-	if (style->conditions != NULL)
+	if (elem_is_set (style, MSTYLE_CONDITIONS) && style->conditions)
 		style->cond_styles = gnm_style_conditions_overlay (style->conditions, style);
 
-	if (style->color.back && !style->color.back->is_auto)
-		hash ^= GPOINTER_TO_UINT (style->color.back);
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	/* ---------------------------------------- */
 
-	if (style->color.pattern && !style->color.pattern->is_auto)
-		hash ^= GPOINTER_TO_UINT (style->color.pattern);
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_COLOR_BACK)) {
+		if (!style->color.back->is_auto)
+			hash ^= GPOINTER_TO_UINT (style->color.back);
+		else
+			hash++;
+	}
+	MIX (hash);
 
-	if (style->color.font && !style->color.font->is_auto)
-		hash ^= GPOINTER_TO_UINT (style->color.font);
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_COLOR_PATTERN)) {
+		if (!style->color.pattern->is_auto)
+			hash ^= GPOINTER_TO_UINT (style->color.pattern);
+		else
+			hash++;
+	}
+	MIX (hash);
+
+	if (elem_is_set (style, MSTYLE_FONT_COLOR)) {
+		if (!style->color.font->is_auto)
+			hash ^= GPOINTER_TO_UINT (style->color.font);
+		else
+			hash++;
+	}
+	MIX (hash);
 
 	for (i = MSTYLE_BORDER_TOP; i <= MSTYLE_BORDER_DIAGONAL; i++) {
-		hash ^= GPOINTER_TO_UINT (style->borders[i - MSTYLE_BORDER_TOP]);
-		hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+		if (elem_is_set (style, i))
+			hash ^= GPOINTER_TO_UINT (style->borders[i - MSTYLE_BORDER_TOP]);
+		else
+			hash++;
+		MIX (hash);
 	}
 
-	hash ^= style->pattern;
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_PATTERN))
+		hash ^= style->pattern;
+	MIX (hash);
 
-	hash ^= GPOINTER_TO_UINT (style->font_detail.name);
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_FONT_NAME))
+		hash ^= GPOINTER_TO_UINT (style->font_detail.name);
+	MIX (hash);
 
-	if (style->font_detail.bold) {
-		hash ^= 0x1379;
-		hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
-	}
+	if (elem_is_set (style, MSTYLE_FONT_BOLD))
+		hash ^= (style->font_detail.bold ? 1 : 2);
+	MIX (hash);
 
-	if (style->font_detail.italic) {
-		hash ^= 0x1379;
-		hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
-	}
+	if (elem_is_set (style, MSTYLE_FONT_ITALIC))
+		hash ^= (style->font_detail.italic ? 1 : 2);
+	MIX (hash);
 
-	hash ^= style->font_detail.underline;
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_FONT_UNDERLINE))
+		hash ^= (style->font_detail.underline ? 1 : 2);
+	MIX (hash);
 
-	if (style->font_detail.strikethrough)
-		hash ^= 0x1379;
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_FONT_STRIKETHROUGH))
+		hash ^= (style->font_detail.strikethrough ? 1 : 2);
+	MIX (hash);
 
-	hash ^= ((int)(style->font_detail.size * 97));
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_FONT_SCRIPT))
+		hash ^= (style->font_detail.script + 0x100);
+	MIX (hash);
 
-	hash ^= GPOINTER_TO_UINT (style->format);
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_FONT_SIZE))
+		hash ^= ((int)(style->font_detail.size * 97));
+	MIX (hash);
 
-	hash ^= style->h_align;
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_FORMAT))
+		hash ^= GPOINTER_TO_UINT (style->format);
+	MIX (hash);
 
-	hash ^= style->v_align;
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_ALIGN_H))
+		hash ^= (style->h_align + 0x100);
+	MIX (hash);
 
-	hash ^= style->indent;
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_ALIGN_V))
+		hash ^= (style->v_align + 0x100);
+	MIX (hash);
 
-	hash ^= style->rotation;
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_INDENT))
+		hash ^= style->indent;
+	MIX (hash);
 
-	hash ^= style->text_dir;
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_ROTATION))
+		hash ^= style->rotation;
+	MIX (hash);
 
-	if (style->wrap_text)
-		hash ^= 0x1379;
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_TEXT_DIR))
+		hash ^= (style->text_dir + 0x100);
+	MIX (hash);
 
-	if (style->shrink_to_fit)
-		hash ^= 0x1379;
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_WRAP_TEXT))
+		hash ^= (style->wrap_text ? 1 : 2);
+	MIX (hash);
 
-	if (style->contents_locked)
-		hash ^= 0x1379;
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_SHRINK_TO_FIT))
+		hash ^= (style->shrink_to_fit ? 1 : 2);
+	MIX (hash);
 
-	if (style->contents_hidden)
-		hash ^= 0x1379;
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_CONTENTS_LOCKED))
+		hash ^= (style->contents_locked ? 1 : 2);
+	MIX (hash);
 
-	style->hash_key_xl = hash;
+	if (elem_is_set (style, MSTYLE_CONTENTS_HIDDEN))
+		hash ^= (style->contents_hidden ? 1 : 2);
+	MIX (hash);
+
+	style->hash_key_xl = (guint32)hash;
 
 	/* From here on, fields are not in MS XL */
 
-	hash ^= GPOINTER_TO_UINT (style->validation);
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_VALIDATION)) {
+		/*
+		 * The hash used must not depend on the expressions inside
+		 * the validation.
+		 */
+		hash ^= (style->validation != NULL ? 1 : 2);
+	}
+	MIX (hash);
 
-	hash ^= GPOINTER_TO_UINT (style->hlink);
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_HLINK))
+		hash ^= GPOINTER_TO_UINT (style->hlink);
+	MIX (hash);
 
-	hash ^= GPOINTER_TO_UINT (style->input_msg);
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_INPUT_MSG))
+		hash ^= GPOINTER_TO_UINT (style->input_msg);
+	MIX (hash);
 
-	hash ^= GPOINTER_TO_UINT (style->conditions);
-	hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
+	if (elem_is_set (style, MSTYLE_CONDITIONS)) {
+		/*
+		 * The hash used must not depend on the expressions inside
+		 * the conditions.
+		 */
+		hash ^= style->conditions
+			? gnm_style_conditions_hash (style->conditions)
+			: 1u;
+	}
+	MIX (hash);
 
-	style->hash_key = hash;
+	style->hash_key = (guint32)hash;
 
 	if (G_UNLIKELY (style->set == 0)) {
 		/*
@@ -268,6 +318,8 @@ gnm_style_update (GnmStyle *style)
 	}
 }
 
+#undef MIX
+
 guint
 gnm_style_hash_XL (gconstpointer style)
 {
@@ -339,6 +391,12 @@ gnm_style_hash (gconstpointer style)
     ? a->conditions == b->conditions					\
     : FALSE)))))))))))))))))))))))))
 
+/*
+ * Note: the above is suboptimal from validation and down.
+ *
+ * We are comparing pointers (which at least safely matches what we do
+ * with the hash), but I think we want proper equality.
+ */
 
 static gboolean
 elem_is_eq (GnmStyle const *a, GnmStyle const *b, GnmStyleElement elem)
@@ -381,7 +439,7 @@ elem_assign_contents (GnmStyle *dst, GnmStyle const *src, GnmStyleElement elem)
 	case MSTYLE_CONTENTS_HIDDEN:	dst->contents_hidden = src->contents_hidden; return;
 	case MSTYLE_VALIDATION:
 		if ((dst->validation = src->validation))
-			validation_ref (dst->validation);
+			gnm_validation_ref (dst->validation);
 		return;
 	case MSTYLE_HLINK:
 		if ((dst->hlink = src->hlink))
@@ -420,20 +478,20 @@ elem_clear_contents (GnmStyle *style, GnmStyleElement elem)
 	case MSTYLE_FORMAT:		go_format_unref (style->format); return;
 	case MSTYLE_VALIDATION:
 		if (style->validation)
-			validation_unref (style->validation);
+			gnm_validation_unref (style->validation);
 		return;
 	case MSTYLE_HLINK:
 		if (style->hlink)
-			g_object_unref (G_OBJECT (style->hlink));
+			g_object_unref (style->hlink);
 		return;
 	case MSTYLE_INPUT_MSG:
 		if (style->input_msg)
-			g_object_unref (G_OBJECT (style->input_msg));
+			g_object_unref (style->input_msg);
 		return;
 	case MSTYLE_CONDITIONS:
 		if (style->conditions) {
 			clear_conditional_merges (style);
-			g_object_unref (G_OBJECT (style->conditions));
+			g_object_unref (style->conditions);
 		}
 		return;
 	default:
@@ -763,6 +821,29 @@ link_border_colors (GnmStyle *style, GnmColor *auto_color, gboolean make_copy)
 	return style;
 }
 
+static void
+gnm_style_linked_sheet_changed (GnmStyle *style)
+{
+	Sheet *sheet = style->linked_sheet;
+
+	if (elem_is_set (style, MSTYLE_VALIDATION) &&
+	    style->validation &&
+	    gnm_validation_get_sheet (style->validation) != sheet) {
+		GnmValidation *new_v = gnm_validation_dup (style->validation);
+		gnm_validation_set_sheet (new_v, sheet);
+		gnm_style_set_validation (style, new_v);
+	}
+
+	if (elem_is_set (style, MSTYLE_CONDITIONS) &&
+	    style->conditions
+	    && gnm_style_conditions_get_sheet (style->conditions) != sheet
+		) {
+		GnmStyleConditions *new_c = gnm_style_conditions_dup (style->conditions);
+		gnm_style_conditions_set_sheet (new_c, sheet);
+		gnm_style_set_conditions (style, new_c);
+	}
+}
+
 /**
  * gnm_style_link_sheet :
  * @style :
@@ -804,6 +885,8 @@ gnm_style_link_sheet (GnmStyle *style, Sheet *sheet)
 	style->linked_sheet = sheet;
 	style->link_count = 1;
 
+	gnm_style_linked_sheet_changed (style);
+
 	d(("link sheet %p = 1\n", style));
 	return style;
 }
@@ -839,6 +922,12 @@ gnm_style_unlink (GnmStyle *style)
 	}
 }
 
+gboolean
+gnm_style_eq (GnmStyle const *a, GnmStyle const *b)
+{
+	return a == b;
+}
+
 gboolean
 gnm_style_equal (GnmStyle const *a, GnmStyle const *b)
 {
@@ -846,10 +935,10 @@ gnm_style_equal (GnmStyle const *a, GnmStyle const *b)
 
 	if (a == b)
 		return TRUE;
-	if (!gnm_style_equal_XL (a, b))
+	if (a->set != b->set || !gnm_style_equal_XL (a, b))
 		return FALSE;
 	UNROLLED_FOR (i = MSTYLE_VALIDATION, i < MSTYLE_ELEMENT_MAX, i++, {
-		if (!ELEM_IS_EQ (a, b, i))
+		if (elem_is_set (a, i) && !ELEM_IS_EQ (a, b, i))
 			return FALSE;
 	});
 
@@ -866,8 +955,12 @@ gnm_style_equal_XL (GnmStyle const *a, GnmStyle const *b)
 
 	if (a == b)
 		return TRUE;
+
+	if ((a->set ^ b->set) & ((1u << MSTYLE_VALIDATION) - 1))
+		return FALSE;
+
 	UNROLLED_FOR (i = MSTYLE_COLOR_BACK, i < MSTYLE_VALIDATION, i++, {
-		if (!ELEM_IS_EQ (a, b, i))
+		if (elem_is_set (a, i) && !ELEM_IS_EQ (a, b, i))
 			return FALSE;
 	});
 	return TRUE;
