@@ -784,23 +784,6 @@ gnm_range_equal (const GnmRange *a, const GnmRange *b)
 	return range_equal (a, b);
 }
 
-guint
-gnm_range_hash (const GnmRange *r)
-{
-	guint res = 0;
-
-	res ^= r->start.col;
-	res *= 17;
-	res ^= r->start.row;
-	res *= 17;
-	res ^= r->end.col;
-	res *= 17;
-	res ^= r->end.row;
-
-	return res;
-}
-
-
 GnmSheetRange *
 gnm_sheet_range_new (Sheet *sheet, GnmRange const *r)
 {
@@ -854,12 +837,6 @@ gnm_sheet_range_equal (const GnmSheetRange *a,
 	return a->sheet == b->sheet && range_equal (&a->range, &b->range);
 }
 
-guint
-gnm_sheet_range_hash (const GnmSheetRange *sr)
-{
-	return gnm_range_hash (&sr->range) ^ sr->sheet->index_in_wb;
-}
-
 gboolean
 gnm_sheet_range_overlap (GnmSheetRange const *a, GnmSheetRange const *b)
 {
