@@ -39,11 +39,45 @@
 typedef GObjectClass GnmStyleConditionsClass;
 struct _GnmStyleConditions {
 	GObject base;
-	GArray *conditions;
+	GPtrArray *conditions;
+	Sheet *sheet;
 };
 
 static GObjectClass *parent_class;
 
+static unsigned
+gnm_style_cond_op_operands (GnmStyleCondOp op)
+{
+	switch (op) {
+	case GNM_STYLE_COND_BETWEEN:
+	case GNM_STYLE_COND_NOT_BETWEEN:
+		return 2;
+
+	case GNM_STYLE_COND_EQUAL:
+	case GNM_STYLE_COND_NOT_EQUAL:
+	case GNM_STYLE_COND_GT:
+	case GNM_STYLE_COND_LT:
+	case GNM_STYLE_COND_GTE:
+	case GNM_STYLE_COND_LTE:
+	case GNM_STYLE_COND_CUSTOM:
+	case GNM_STYLE_COND_CONTAINS_STR:
+	case GNM_STYLE_COND_NOT_CONTAINS_STR:
+	case GNM_STYLE_COND_BEGINS_WITH_STR:
+	case GNM_STYLE_COND_NOT_BEGINS_WITH_STR:
+	case GNM_STYLE_COND_ENDS_WITH_STR:
+	case GNM_STYLE_COND_NOT_ENDS_WITH_STR:
+		return 1;
+
+	case GNM_STYLE_COND_CONTAINS_ERR:
+	case GNM_STYLE_COND_NOT_CONTAINS_ERR:
+	case GNM_STYLE_COND_CONTAINS_BLANKS:
+	case GNM_STYLE_COND_NOT_CONTAINS_BLANKS:
+		return 0;
+	}
+	g_assert_not_reached ();
+}
+
+
 /**
  * gnm_style_cond_is_valid :
  * @cond : #GnmStyleCond
@@ -53,47 +87,132 @@ static GObjectClass *parent_class;
 gboolean
 gnm_style_cond_is_valid (GnmStyleCond const *cond)
 {
+	unsigned ui, N;
+
 	g_return_val_if_fail (cond != NULL, FALSE);
 
 	if (cond->overlay == NULL) return FALSE;
-	if ((cond->texpr[0] != NULL) ^
-	    (cond->op != GNM_STYLE_COND_CONTAINS_ERR &&
-	     cond->op != GNM_STYLE_COND_NOT_CONTAINS_ERR &&
-	     cond->op != GNM_STYLE_COND_CONTAINS_BLANKS &&
-	     cond->op != GNM_STYLE_COND_NOT_CONTAINS_BLANKS)) return FALSE;
-	if ((cond->texpr[1] != NULL) ^
-	    (cond->op == GNM_STYLE_COND_BETWEEN ||
-	     cond->op == GNM_STYLE_COND_NOT_BETWEEN))
-	    return FALSE;
+
+	N = gnm_style_cond_op_operands (cond->op);
+	for (ui = 0; ui < G_N_ELEMENTS (cond->deps); ui++) {
+		gboolean need = (ui < N);
+		gboolean have = (cond->deps[ui].texpr != NULL);
+		if (have != need)
+			return FALSE;		    
+	}
+
 	return TRUE;
 }
 
-static void
-cond_unref (GnmStyleCond const *cond)
+GnmStyleCond *
+gnm_style_cond_new (GnmStyleCondOp op, Sheet *sheet)
 {
+	GnmStyleCond *res;
+	unsigned ui;
+
+	g_return_val_if_fail (IS_SHEET (sheet), NULL);
+
+	res = g_new0 (GnmStyleCond, 1);
+	res->op = op;
+	for (ui = 0; ui < 2; ui++)
+		dependent_managed_init (&res->deps[ui], sheet);
+	return res;
+}
+
+GnmStyleCond *
+gnm_style_cond_dup (GnmStyleCond const *src)
+{
+	GnmStyleCond *dst;
+	unsigned ui;
+
+	g_return_val_if_fail (src != NULL, NULL);
+
+	dst = gnm_style_cond_new (src->op, gnm_style_cond_get_sheet (src));
+	gnm_style_cond_set_overlay (dst, src->overlay);
+	for (ui = 0; ui < 2; ui++)
+		gnm_style_cond_set_expr (dst, src->deps[ui].texpr, ui);
+
+	return dst;
+}
+
+void
+gnm_style_cond_free (GnmStyleCond *cond)
+{
+	unsigned ui;
+
+	g_return_if_fail (cond != NULL);
+
 	/* Be very delicate, this is called for invalid conditions too */
 	if (cond->overlay)
 		gnm_style_unref (cond->overlay);
-	if (cond->texpr[0])
-		gnm_expr_top_unref (cond->texpr[0]);
-	if (cond->texpr[1])
-		gnm_expr_top_unref (cond->texpr[1]);
+	for (ui = 0; ui < 2; ui++)
+		gnm_style_cond_set_expr (cond, NULL, ui);
+
+	g_free (cond);
 }
 
+Sheet *
+gnm_style_cond_get_sheet (GnmStyleCond const *cond)
+{
+	g_return_val_if_fail (cond != NULL, NULL);
+	return cond->deps[0].sheet;
+}
+
+void
+gnm_style_cond_set_sheet (GnmStyleCond *cond, Sheet *sheet)
+{
+	int ui;
+
+	g_return_if_fail (cond != NULL);
+	g_return_if_fail (IS_SHEET (sheet));
+
+	for (ui = 0; ui < 2; ui++)
+		dependent_managed_set_sheet (&cond->deps[ui], sheet);
+}
+
+GnmExprTop const *
+gnm_style_cond_get_expr (GnmStyleCond const *cond, unsigned idx)
+{
+	g_return_val_if_fail (cond != NULL, NULL);
+	g_return_val_if_fail (idx < G_N_ELEMENTS (cond->deps), NULL);
+
+	return cond->deps[idx].texpr;
+}
+
+void
+gnm_style_cond_set_expr (GnmStyleCond *cond,
+			 GnmExprTop const *texpr,
+			 unsigned idx)
+{
+	g_return_if_fail (cond != NULL);
+	g_return_if_fail (idx < G_N_ELEMENTS (cond->deps));
+
+	dependent_managed_set_expr (&cond->deps[idx], texpr);
+}
+
+void
+gnm_style_cond_set_overlay (GnmStyleCond *cond, GnmStyle *overlay)
+{
+	g_return_if_fail (cond != NULL);
+
+	if (overlay)
+		gnm_style_ref (overlay);
+	if (cond->overlay)
+		gnm_style_unref (cond->overlay);
+	cond->overlay = overlay;
+}
+
+
 static void
 gnm_style_conditions_finalize (GObject *obj)
 {
 	GnmStyleConditions *sc = (GnmStyleConditions *)obj;
 
-	if (sc->conditions != NULL) {
-		int i = sc->conditions->len;
-		while (i-- > 0)
-			cond_unref (&g_array_index (sc->conditions, GnmStyleCond, i));
-		g_array_free (sc->conditions, TRUE);
-		sc->conditions = NULL;
-	}
+	while (sc->conditions)
+		gnm_style_conditions_delete (sc, sc->conditions->len - 1);
 	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
+
 static void
 gnm_style_conditions_init (GnmStyleConditions *sc)
 {
@@ -118,40 +237,96 @@ static GSF_CLASS (GnmStyleConditions, gnm_style_conditions,
  * Returns a GnmStyleConditions that the caller is resoinsible for.
  **/
 GnmStyleConditions  *
-gnm_style_conditions_new (void)
+gnm_style_conditions_new (Sheet *sheet)
 {
-	return g_object_new (gnm_style_conditions_get_type (), NULL);
+	GnmStyleConditions *res;
+	g_return_val_if_fail (IS_SHEET (sheet), NULL);
+
+	res = g_object_new (gnm_style_conditions_get_type (), NULL);
+	res->sheet = sheet;
+	return res;
 }
 
 GnmStyleConditions *
-gnm_style_conditions_dup  (GnmStyleConditions const *cond)
+gnm_style_conditions_dup (GnmStyleConditions const *sc)
 {
-	GnmStyleConditions  *dup;
-	GArray const *ga;
-	if (cond == NULL)
+	GnmStyleConditions *dup;
+	GPtrArray const *ga;
+	if (sc == NULL)
 		return NULL;
 
-	dup =  gnm_style_conditions_new ();
-	ga = gnm_style_conditions_details (cond);
+	dup = gnm_style_conditions_new (gnm_style_conditions_get_sheet (sc));
+	ga = gnm_style_conditions_details (sc);
 	if (ga != NULL) {
 		guint i;
-		GArray *ga_dup = g_array_sized_new (FALSE, FALSE, sizeof (GnmStyleCond),
-						    ga->len);
+		GPtrArray *ga_dup = g_ptr_array_sized_new (ga->len);
 		for (i = 0; i < ga->len; i++) {
-			GnmStyleCond gsc = g_array_index(ga, GnmStyleCond, i);
-
-			gnm_style_ref (gsc.overlay);
-			if (gsc.texpr[0])
-				gnm_expr_top_ref (gsc.texpr[0]);
-			if (gsc.texpr[1])
-				gnm_expr_top_ref (gsc.texpr[1]);
-			g_array_append_val (ga_dup, gsc);
+			GnmStyleCond *cond = g_ptr_array_index (ga, i);
+			g_ptr_array_add (ga_dup, gnm_style_cond_dup (cond));
 		}
 		dup->conditions = ga_dup;
 	}
 	return dup;
 }
 
+#define MIX(H) do {				\
+  H *= G_GUINT64_CONSTANT(123456789012345);	\
+  H ^= (H >> 31);				\
+} while (0)
+
+guint32
+gnm_style_conditions_hash (GnmStyleConditions const *sc)
+{
+	guint64 hash = 42;
+	GPtrArray const *ga;
+	unsigned ui;
+
+	/*
+	 * Note: this hash must not depend on the expressions stored
+	 * in ->deps.  And probably not on the sheet either.
+	 */
+
+	g_return_val_if_fail (sc != NULL, 0u);
+
+	ga = gnm_style_conditions_details (sc);
+	for (ui = 0; ui < (ga ? ga->len : 0u); ui++) {
+		GnmStyleCond *cond = g_ptr_array_index (ga, ui);
+		if (cond->overlay)
+			hash ^= gnm_style_hash_XL (cond->overlay);
+		MIX (hash);
+		hash ^= cond->op;
+		MIX (hash);
+	}
+
+	return hash;
+}
+
+#undef MIX
+
+Sheet *
+gnm_style_conditions_get_sheet (GnmStyleConditions const *sc)
+{
+	g_return_val_if_fail (sc != NULL, NULL);
+	return sc->sheet;
+}
+
+void
+gnm_style_conditions_set_sheet (GnmStyleConditions *sc, Sheet *sheet)
+{
+	GPtrArray const *ga;
+	unsigned ui;
+
+	g_return_if_fail (sc != NULL);
+	g_return_if_fail (IS_SHEET (sheet));
+
+	sc->sheet = sheet;
+	ga = gnm_style_conditions_details (sc);
+	for (ui = 0; ga && ui < ga->len; ui++) {
+		GnmStyleCond *cond = g_ptr_array_index (ga, ui);
+		gnm_style_cond_set_sheet (cond, sheet);
+	}
+}
+
 
 /**
  * gnm_style_conditions_details :
@@ -159,7 +334,7 @@ gnm_style_conditions_dup  (GnmStyleConditions const *cond)
  *
  * Returns an array of GnmStyleCond which should not be modified.
  **/
-GArray const	*
+GPtrArray const *
 gnm_style_conditions_details (GnmStyleConditions const *sc)
 {
 	g_return_val_if_fail (sc != NULL, NULL);
@@ -177,34 +352,48 @@ gnm_style_conditions_details (GnmStyleConditions const *sc)
  **/
 void
 gnm_style_conditions_insert (GnmStyleConditions *sc,
-			     GnmStyleCond const *cond, int pos)
+			     GnmStyleCond const *cond_, int pos)
 {
-	g_return_if_fail (cond != NULL);
+	GnmStyleCond *cond;
 
-	if (sc == NULL || !gnm_style_cond_is_valid (cond)) {
-		cond_unref (cond); /* be careful not to leak */
-		return;
-	}
+	g_return_if_fail (sc != NULL);
+	g_return_if_fail (cond_ != NULL);
+
+	g_return_if_fail (gnm_style_cond_is_valid (cond_));
+
+	g_return_if_fail (gnm_style_conditions_get_sheet (sc) ==
+			  gnm_style_cond_get_sheet (cond_));
 
 	if (sc->conditions == NULL)
-		sc->conditions = g_array_new (FALSE, FALSE, sizeof (GnmStyleCond));
+		sc->conditions = g_ptr_array_new ();
 
-	if (pos < 0)
-		g_array_append_val (sc->conditions, *cond);
-	else
-		g_array_insert_val (sc->conditions, pos, *cond);
+	cond = gnm_style_cond_dup (cond_);
+	g_ptr_array_add (sc->conditions, cond);
+	if (pos >= 0) {
+		int i;
+
+		for (i = sc->conditions->len - 1;
+		     i > pos;
+		     i--)
+			g_ptr_array_index (sc->conditions, i) =
+				g_ptr_array_index (sc->conditions, i - 1);
+		g_ptr_array_index (sc->conditions, pos) = cond;
+	}
 }
 
 void
-gnm_style_conditions_delete  (GnmStyleConditions *sc,
-			      guint pos)
+gnm_style_conditions_delete (GnmStyleConditions *sc, guint pos)
 {
 	g_return_if_fail (sc != NULL);
 	g_return_if_fail (sc->conditions != NULL);
 	g_return_if_fail (sc->conditions->len > pos);
 
-	cond_unref (&g_array_index (sc->conditions, GnmStyleCond, pos));
-	g_array_remove_index (sc->conditions, pos);
+	gnm_style_cond_free (g_ptr_array_index (sc->conditions, pos));
+	if (sc->conditions->len <= 1) {
+		g_ptr_array_free (sc->conditions, TRUE);
+		sc->conditions = NULL;
+	} else
+		g_ptr_array_remove_index (sc->conditions, pos);
 }
 
 
@@ -213,8 +402,6 @@ gnm_style_conditions_overlay (GnmStyleConditions const *sc,
 			      GnmStyle const *base)
 {
 	GPtrArray *res;
-	GnmStyle const *overlay;
-	GnmStyle *merge;
 	unsigned i;
 
 	g_return_val_if_fail (sc != NULL, NULL);
@@ -222,8 +409,10 @@ gnm_style_conditions_overlay (GnmStyleConditions const *sc,
 
 	res = g_ptr_array_sized_new (sc->conditions->len);
 	for (i = 0 ; i < sc->conditions->len; i++) {
-		overlay = g_array_index (sc->conditions, GnmStyleCond, i).overlay;
-		merge = gnm_style_new_merged (base, overlay);
+		GnmStyleCond const *cond =
+			g_ptr_array_index (sc->conditions, i);
+		GnmStyle const *overlay = cond->overlay;
+		GnmStyle *merge = gnm_style_new_merged (base, overlay);
 		/* We only draw a background colour is the pattern != 0 */
 		if (merge->pattern == 0 &&
 		     elem_is_set (overlay, MSTYLE_COLOR_BACK) &&
@@ -247,8 +436,7 @@ gnm_style_conditions_eval (GnmStyleConditions const *sc, GnmEvalPos const *ep)
 	unsigned i;
 	gboolean use_this = FALSE;
 	GnmValue *val = NULL;
-	GArray const *conds;
-	GnmStyleCond const *cond;
+	GPtrArray const *conds;
 	GnmParsePos pp;
 	GnmCell const *cell = sheet_cell_get (ep->sheet, ep->eval.col, ep->eval.row);
 	GnmValue const *cv = cell ? cell->value : NULL;
@@ -260,7 +448,7 @@ gnm_style_conditions_eval (GnmStyleConditions const *sc, GnmEvalPos const *ep)
 	conds = sc->conditions;
 	parse_pos_init_evalpos (&pp, ep);
 	for (i = 0 ; i < conds->len ; i++) {
-		cond = &g_array_index (conds, GnmStyleCond, i);
+		GnmStyleCond const *cond = g_ptr_array_index (conds, i);
 
 		if (cond->op == GNM_STYLE_COND_CONTAINS_ERR)
 			use_this = (cv != NULL) && VALUE_IS_ERROR (cv);
@@ -282,7 +470,7 @@ gnm_style_conditions_eval (GnmStyleConditions const *sc, GnmEvalPos const *ep)
 				}
 			}
 		} else {
-			val = gnm_expr_top_eval (cond->texpr[0], ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
+			val = gnm_expr_top_eval (cond->deps[0].texpr, ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
 			if (cond->op == GNM_STYLE_COND_CUSTOM) {
 				use_this = value_get_as_bool (val, NULL);
 #if 0
@@ -304,7 +492,7 @@ gnm_style_conditions_eval (GnmStyleConditions const *sc, GnmEvalPos const *ep)
 						break;
 					}
 					value_release (val);
-					val = gnm_expr_top_eval (cond->texpr[1], ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
+					val = gnm_expr_top_eval (cond->deps[1].texpr, ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
 					diff = value_compare (cv, val, TRUE);
 					/* fall through */
 
@@ -315,7 +503,7 @@ gnm_style_conditions_eval (GnmStyleConditions const *sc, GnmEvalPos const *ep)
 					if (diff == IS_LESS)
 						break;
 					value_release (val);
-					val = gnm_expr_top_eval (cond->texpr[1], ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
+					val = gnm_expr_top_eval (cond->deps[1].texpr, ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
 					diff = value_compare (cv, val, TRUE);
 					/* fall through */
 				case GNM_STYLE_COND_LTE:	use_this = (diff == IS_LESS || diff == IS_EQUAL); break;
