@@ -335,7 +335,7 @@ gnm_align_h_get_type (void)
 			{GNM_HALIGN_CENTER, "GNM_HALIGN_CENTER", "center"},
 			{GNM_HALIGN_FILL, "GNM_HALIGN_FILL", "fill"},
 			{GNM_HALIGN_JUSTIFY, "GNM_HALIGN_JUSTIFY", "justify"},
-			{GNM_HALIGN_CENTER_ACROSS_SELECTION, 
+			{GNM_HALIGN_CENTER_ACROSS_SELECTION,
 			 "GNM_HALIGN_CENTER_ACROSS_SELECTION",
 			 "across-selection"},
 			{GNM_HALIGN_DISTRIBUTED,
@@ -358,7 +358,7 @@ gnm_align_v_get_type (void)
 			{GNM_VALIGN_BOTTOM, "GNM_VALIGN_BOTTOM", "bottom"},
 			{GNM_VALIGN_CENTER, "GNM_VALIGN_CENTER", "center"},
 			{GNM_VALIGN_JUSTIFY, "GNM_VALIGN_JUSTIFY", "justify"},
-			{GNM_VALIGN_DISTRIBUTED, 
+			{GNM_VALIGN_DISTRIBUTED,
 			 "GNM_VALIGN_DISTRIBUTED", "distributed"},
 			{ 0, NULL, NULL }
 		};
@@ -452,7 +452,8 @@ gnm_font_init (void)
 }
 
 static void
-delete_neg_font (GnmFont *sf, gpointer value, gpointer user_data)
+delete_neg_font (GnmFont *sf, G_GNUC_UNUSED gpointer value,
+		 G_GNUC_UNUSED gpointer user_data)
 {
 	g_object_unref (sf->context);
 	g_free (sf->font_name);
@@ -460,7 +461,7 @@ delete_neg_font (GnmFont *sf, gpointer value, gpointer user_data)
 }
 
 static void
-list_cached_fonts (GnmFont *font, gpointer value, GSList **lp)
+list_cached_fonts (GnmFont *font, G_GNUC_UNUSED gpointer value, GSList **lp)
 {
 	*lp = g_slist_prepend (*lp, font);
 }
