@@ -11,6 +11,7 @@
 #include "workbook-view.h"
 #include "workbook-control.h"
 #include "gnm-marshalers.h"
+#include "dao.h"
 #include <gsf/gsf-impl-utils.h>
 #include <gsf/gsf-output-stdio.h>
 #include <glib/gi18n-lib.h>
@@ -43,6 +44,23 @@ gnm_solver_debug (void)
 	return debug;
 }
 
+static char *
+gnm_solver_cell_name (GnmCell const *cell, Sheet *origin)
+{
+	GnmConventionsOut out;
+	GnmCellRef cr;
+	GnmParsePos pp;
+
+	gnm_cellref_init (&cr, cell->base.sheet,
+			  cell->pos.col, cell->pos.row,
+			  TRUE);
+	out.accum = g_string_new (NULL);
+	out.pp = parse_pos_init_sheet (&pp, origin);
+	out.convs = origin->convs;
+	cellref_as_string (&out, &cr, cell->base.sheet == origin);
+	return g_string_free (out.accum, FALSE);
+}
+
 /* ------------------------------------------------------------------------- */
 
 GType
@@ -182,13 +200,13 @@ gnm_solver_constraint_valid (GnmSolverConstraint const *c,
 		if (rhs == NULL)
 			return FALSE;
 		if (rhs->type == VALUE_CELLRANGE) {
-			GnmRange rl, rr;
+			GnmSheetRange srl, srr;
 
-			range_init_value (&rl, lhs);
-			range_init_value (&rr, rhs);
+			gnm_sheet_range_from_value (&srl, lhs);
+			gnm_sheet_range_from_value (&srr, rhs);
 
-			if (range_width (&rl) != range_width (&rr) ||
-			    range_height (&rl) != range_height (&rr))
+			if (range_width (&srl.range) != range_width (&srr.range) ||
+			    range_height (&srl.range) != range_height (&srr.range))
 				return FALSE;
 		} else if (VALUE_IS_FLOAT (rhs)) {
 			/* Nothing */
@@ -260,7 +278,7 @@ gnm_solver_constraint_get_part (GnmSolverConstraint const *c,
 				GnmCell **lhs, gnm_float *cl,
 				GnmCell **rhs, gnm_float *cr)
 {
-	GnmRange r;
+	GnmSheetRange sr;
 	int h, w, dx, dy;
 	GnmValue const *vl, *vr;
 
@@ -275,9 +293,9 @@ gnm_solver_constraint_get_part (GnmSolverConstraint const *c,
 	vl = gnm_solver_constraint_get_lhs (c);
 	vr = gnm_solver_constraint_get_rhs (c);
 
-	range_init_value (&r, vl);
-	w = range_width (&r);
-	h = range_height (&r);
+	gnm_sheet_range_from_value (&sr, vl);
+	w = range_width (&sr.range);
+	h = range_height (&sr.range);
 
 	dy = i / w;
 	dx = i % w;
@@ -285,20 +303,21 @@ gnm_solver_constraint_get_part (GnmSolverConstraint const *c,
 		return FALSE;
 
 	if (lhs)
-		*lhs = sheet_cell_get (sp->sheet,
-				       r.start.col + dx, r.start.row + dy);
+		*lhs = sheet_cell_get (eval_sheet (sr.sheet, sp->sheet),
+				       sr.range.start.col + dx,
+				       sr.range.start.row + dy);
 
-	if (gnm_solver_constraint_has_rhs (c)) {
-		if (VALUE_IS_FLOAT (vr)) {
-			if (cr)
-				*cr = value_get_as_float (vr);
-		} else {
-			range_init_value (&r, vr);
-			if (rhs)
-				*rhs = sheet_cell_get (sp->sheet,
-						       r.start.col + dx,
-						       r.start.row + dy);
-		}
+	if (!gnm_solver_constraint_has_rhs (c)) {
+		/* Nothing */
+	} else if (VALUE_IS_FLOAT (vr)) {
+		if (cr)
+			*cr = value_get_as_float (vr);
+	} else {
+		gnm_sheet_range_from_value (&sr, vr);
+		if (rhs)
+			*rhs = sheet_cell_get (eval_sheet (sr.sheet, sp->sheet),
+					       sr.range.start.col + dx,
+					       sr.range.start.row + dy);
 	}
 
 	return TRUE;
@@ -359,13 +378,17 @@ gnm_solver_constraint_as_str (GnmSolverConstraint const *c, Sheet *sheet)
 	const char * const type_str[] =	{
 		"\xe2\x89\xa4" /* "<=" */,
 		"\xe2\x89\xa5" /* ">=" */,
-		"=", "Int", "Bool"
+		"=",
+		N_("Int"),
+		N_("Bool")
 	};
+	const char *type = type_str[c->type];
+	gboolean translate = (c->type >= GNM_SOLVER_INTEGER);
 	GString *buf = g_string_new (NULL);
 
 	gnm_solver_constraint_side_as_str (c, sheet, buf, TRUE);
 	g_string_append_c (buf, ' ');
-	g_string_append (buf, type_str[c->type]);
+	g_string_append (buf, translate ? _(type) : type);
 	if (gnm_solver_constraint_has_rhs (c)) {
 		g_string_append_c (buf, ' ');
 		gnm_solver_constraint_side_as_str (c, sheet, buf, FALSE);
@@ -573,11 +596,13 @@ gnm_solver_param_valid (GnmSolverParameters const *sp, GError **err)
 	if (!gnm_cell_has_expr (target_cell) ||
 	    target_cell->value == NULL ||
 	    !VALUE_IS_FLOAT (target_cell->value)) {
+		char *tcname = gnm_solver_cell_name (target_cell, sp->sheet);
 		g_set_error (err,
 			     go_error_invalid (),
 			     0,
 			     _("Target cell, %s, must contain a formula that evaluates to a number"),
-			     cell_name (target_cell));
+			     tcname);
+		g_free (tcname);
 		return FALSE;
 	}
 
@@ -592,11 +617,13 @@ gnm_solver_param_valid (GnmSolverParameters const *sp, GError **err)
 	for (l = input_cells; l; l = l->next) {
 		GnmCell *cell = l->data;
 		if (gnm_cell_has_expr (cell)) {
+			char *cname = gnm_solver_cell_name (cell, sp->sheet);
 			g_set_error (err,
 				     go_error_invalid (),
 				     0,
 				     _("Input cell %s contains a formula"),
-				     cell_name (cell));
+				     cname);
+			g_free (cname);
 			g_slist_free (input_cells);
 			return FALSE;
 		}
@@ -1229,6 +1256,378 @@ gnm_solver_saveas (GnmSolver *solver, WorkbookControl *wbc,
 	return TRUE;
 }
 
+static gboolean
+cell_in_cr (GnmCell const *cell, GnmSheetRange *sr, gboolean follow,
+	    int *px, int *py)
+{
+	if (!cell)
+		return FALSE;
+
+	if (sr->sheet != cell->base.sheet ||
+	    !range_contains (&sr->range, cell->pos.col, cell->pos.row)) {
+		/* If the expression is just =X42 thenm look at X42 instead.
+		   This is because the mps loader uses such a level of
+		   indirection.  Note: we follow only one such step.  */
+		GnmCellRef const *cr = gnm_expr_top_get_cellref (cell->base.texpr);
+		GnmCellRef cr2;
+		GnmCell const *new_cell;
+		GnmEvalPos ep;
+
+		if (!cr)
+			return FALSE;
+
+		eval_pos_init_cell (&ep, cell);
+		gnm_cellref_make_abs (&cr2, cr, &ep);
+		new_cell = sheet_cell_get (eval_sheet (cr2.sheet, cell->base.sheet),
+					   cr2.col, cr2.row);
+		return cell_in_cr (new_cell, sr, FALSE, px, py);
+
+	}
+
+	*px = cell->pos.col - sr->range.start.col;
+	*py = cell->pos.row - sr->range.start.row;
+	return TRUE;
+}
+
+static gboolean
+cell_is_constant (GnmCell const *cell, gnm_float *pc)
+{
+	if (!cell)
+		return TRUE;
+
+	if (cell->base.texpr)
+		return FALSE;
+
+	*pc = value_get_as_float (cell->value);
+	return TRUE;
+}
+
+#define SET_LOWER(l_)						\
+  do {								\
+	  (*pmin)[y * w + x] = MAX ((*pmin)[y * w + x], (l_));	\
+  } while (0)
+
+#define SET_UPPER(l_)						\
+  do {								\
+	  (*pmax)[y * w + x] = MIN ((*pmax)[y * w + x], (l_));	\
+  } while (0)
+
+
+
+static void
+gnm_solver_get_limits (GnmSolver *solver, gnm_float **pmin, gnm_float **pmax)
+{
+	GnmValue const *vinput;
+	GnmSolverParameters *params = solver->params;
+	int x, y, w, h;
+	GnmSheetRange sr;
+	GSList *l;
+
+	*pmin = *pmax = NULL;
+
+	vinput = gnm_solver_param_get_input (params);
+	if (!vinput) return;
+		
+	gnm_sheet_range_from_value (&sr, vinput);
+	if (!sr.sheet) sr.sheet = params->sheet;
+	h = range_height (&sr.range);
+	w = range_width (&sr.range);
+
+	*pmin = g_new (gnm_float, h * w);
+	*pmax = g_new (gnm_float, h * w);
+
+	for (x = 0; x < w; x++) {
+		for (y = 0; y < h; y++) {
+			(*pmin)[y * w + x] = params->options.assume_non_negative ? 0 : gnm_ninf;
+			(*pmax)[y * w + x] = gnm_pinf;
+		}
+	}
+
+	for (l = params->constraints; l; l = l->next) {
+		GnmSolverConstraint *c = l->data;
+		int i;
+		gnm_float cl, cr;
+		GnmCell *lhs, *rhs;
+
+		for (i = 0;
+		     gnm_solver_constraint_get_part (c, params, i,
+						     &lhs, &cl,
+						     &rhs, &cr);
+		     i++) {
+			if (!cell_in_cr (lhs, &sr, TRUE, &x, &y))
+				continue;
+			if (!cell_is_constant (rhs, &cr))
+				continue;
+
+			switch (c->type) {
+			case GNM_SOLVER_INTEGER:
+				break;
+			case GNM_SOLVER_BOOLEAN:
+				SET_LOWER (0.0);
+				SET_UPPER (1.0);
+				break;
+			case GNM_SOLVER_LE:
+				SET_UPPER (cr);
+				break;
+			case GNM_SOLVER_GE:
+				SET_LOWER (cr);
+				break;
+			case GNM_SOLVER_EQ:
+				SET_LOWER (cr);
+				SET_UPPER (cr);
+				break;
+
+			default:
+				g_assert_not_reached ();
+				break;
+			}
+		}
+	}
+
+}
+
+#undef SET_LOWER
+#undef SET_UPPER
+
+#define ADD_HEADER(txt_) do {			\
+	dao_set_bold (dao, 0, R, 0, R);		\
+	dao_set_cell (dao, 0, R, (txt_));	\
+	R++;					\
+} while (0)
+
+#define AT_LIMIT(s_,l_) \
+  (gnm_finite (l_) ? gnm_abs ((s_) - (l_)) <= (gnm_abs ((s_)) + gnm_abs ((l_))) / 1e10 : (s_) == (l_))
+
+#define MARK_BAD(col_)						\
+  do {								\
+	  int c = (col_);					\
+	  dao_set_colors (dao, c, R, c, R,			\
+			  style_color_new_i8 (255, 0, 0),	\
+			  NULL);				\
+  } while (0)
+
+
+static void
+add_value_or_special (data_analysis_output_t *dao, int col, int row,
+		      gnm_float x)
+{
+	if (gnm_finite (x))
+		dao_set_cell_float (dao, col, row, x);
+	else {
+		dao_set_cell (dao, col, row, "-");
+		dao_set_align (dao, col, row, col, row,
+			       GNM_HALIGN_CENTER, GNM_VALIGN_TOP);
+	}
+}
+
+void
+gnm_solver_create_report (GnmSolver *solver, const char *name)
+{
+	GnmSolverParameters *params = solver->params;
+	int R = 0;
+	GnmValue const *vinput;
+	data_analysis_output_t *dao;
+	GSList *l;
+
+	dao = dao_init_new_sheet (NULL);
+	dao->sheet = params->sheet;
+	dao_prepare_output (NULL, dao, name);
+
+	/* ---------------------------------------- */
+
+	{
+		char *tmp;
+
+		ADD_HEADER (_("Target"));
+		dao_set_cell (dao, 1, R, _("Cell"));
+		dao_set_cell (dao, 2, R, _("Value"));
+		dao_set_cell (dao, 3, R, _("Type"));
+		dao_set_cell (dao, 4, R, _("Status"));
+		R++;
+
+		tmp = gnm_solver_cell_name
+			(gnm_solver_param_get_target_cell (params),
+			 params->sheet);
+		dao_set_cell (dao, 1, R, tmp);
+		g_free (tmp);
+
+		dao_set_cell_float (dao, 2, R, solver->result->value);
+
+		switch (params->problem_type) {
+		case GNM_SOLVER_MINIMIZE:
+			dao_set_cell (dao, 3, R, _("Minimize"));
+			break;
+		case GNM_SOLVER_MAXIMIZE:
+			dao_set_cell (dao, 3, R, _("Maximize"));
+			break;
+		}
+
+		switch (solver->result->quality) {
+		default:
+			break;
+		case GNM_SOLVER_RESULT_FEASIBLE:
+			dao_set_cell (dao, 4, R, _("Feasible"));
+			break;
+		case GNM_SOLVER_RESULT_OPTIMAL:
+			dao_set_cell (dao, 4, R, _("Optimal"));
+			break;
+		}
+
+		R++;
+		R++;
+	}
+
+	/* ---------------------------------------- */
+
+	vinput = gnm_solver_param_get_input (params);
+	if (vinput) {
+		int x, y, w, h;
+		GnmSheetRange sr;
+		gnm_float *pmin, *pmax;
+
+		ADD_HEADER (_("Variables"));
+
+		dao_set_cell (dao, 1, R, _("Cell"));
+		dao_set_cell (dao, 2, R, _("Value"));
+		dao_set_cell (dao, 3, R, _("Lower"));
+		dao_set_cell (dao, 4, R, _("Upper"));
+		dao_set_cell (dao, 5, R, _("Slack"));
+		R++;
+
+		gnm_sheet_range_from_value (&sr, vinput);
+		if (!sr.sheet) sr.sheet = params->sheet;
+		h = range_height (&sr.range);
+		w = range_width (&sr.range);
+
+		gnm_solver_get_limits (solver, &pmin, &pmax);
+
+		for (x = 0; x < w; x++) {
+			for (y = 0; y < h; y++) {
+				GnmCell *cell = sheet_cell_fetch
+					(sr.sheet,
+					 sr.range.start.col + x,
+					 sr.range.start.row + y);
+				gnm_float m = pmin[y * w + x];
+				gnm_float M = pmax[y * w + x];
+				GnmValue const *vs = value_area_fetch_x_y (solver->result->solution, x, y, NULL);
+				gnm_float s = value_get_as_float (vs);
+				gnm_float slack = MIN (s - m, M - s);
+
+				char *cname = gnm_solver_cell_name (cell, params->sheet);
+				dao_set_cell (dao, 1, R, cname);
+				g_free (cname);
+				dao_set_cell_value (dao, 2, R, value_dup (vs));
+				add_value_or_special (dao, 3, R, m);
+				add_value_or_special (dao, 4, R, M);
+
+				add_value_or_special (dao, 5, R, slack);
+				if (slack < 0)
+					MARK_BAD (5);
+
+				if (AT_LIMIT (s, m) || AT_LIMIT (s, M))
+					dao_set_cell (dao, 6, R, _("At limit"));
+
+
+				if (s < m || s > M) {
+					dao_set_cell (dao, 7, R, _("Outside bounds"));
+					MARK_BAD (7);
+				}
+
+				R++;
+			}
+		}
+
+		g_free (pmin);
+		g_free (pmax);
+		R++;
+	}
+
+	/* ---------------------------------------- */
+
+	ADD_HEADER (_("Constraints"));
+
+	if (params->constraints) {
+		dao_set_cell (dao, 1, R, _("Condition"));
+		dao_set_cell (dao, 2, R, _("Value"));
+		dao_set_cell (dao, 3, R, _("Limit"));
+		dao_set_cell (dao, 4, R, _("Slack"));
+	} else {
+		dao_set_cell (dao, 1, R, _("No constraints"));
+	}
+	R++;
+
+	for (l = params->constraints; l; l = l->next) {
+		GnmSolverConstraint *c = l->data;
+		int i;
+		gnm_float cl, cr;
+		GnmCell *lhs, *rhs;
+
+		for (i = 0;
+		     gnm_solver_constraint_get_part (c, params, i,
+						     &lhs, &cl,
+						     &rhs, &cr);
+		     i++) {
+			gnm_float slack = 0;
+			char *ctxt = gnm_solver_constraint_as_str (c, params->sheet);
+			dao_set_cell (dao, 1, R, ctxt);
+			g_free (ctxt);
+
+			if (lhs) {
+				gnm_cell_eval (lhs);
+				cl = value_get_as_float (lhs->value);
+			}
+			if (rhs) {
+				gnm_cell_eval (rhs);
+				cr = value_get_as_float (rhs->value);
+			}
+
+			switch (c->type) {
+			case GNM_SOLVER_INTEGER: {
+				gnm_float c = gnm_fake_round (cl);
+				slack = 0 - gnm_abs (c - cl);
+				break;
+			}
+			case GNM_SOLVER_BOOLEAN: {
+				gnm_float c = (cl > 0.5 ? 1 : 0);
+				slack = 0 - gnm_abs (c - cl);
+				break;
+			}
+			case GNM_SOLVER_LE:
+				slack = cr - cl;
+				break;
+			case GNM_SOLVER_GE:
+				slack = cl - cr;
+				break;
+			case GNM_SOLVER_EQ:
+				slack = 0 - gnm_abs (cl - cr);
+				break;
+			default:
+				g_assert_not_reached ();
+			}
+
+			add_value_or_special (dao, 2, R, cl);
+			if (rhs)
+				add_value_or_special (dao, 3, R, cr);
+
+			add_value_or_special (dao, 4, R, slack);
+			if (slack < 0)
+				MARK_BAD (4);
+
+			R++;
+		}
+	}
+
+	/* ---------------------------------------- */
+
+	dao_redraw_respan (dao);
+
+	dao_free (dao);
+}
+
+#undef AT_LIMIT
+#undef ADD_HEADER
+#undef MARK_BAD
+
 static void
 gnm_solver_class_init (GObjectClass *object_class)
 {
@@ -1601,11 +2000,11 @@ gnm_sub_solver_locate_binary (const char *binary, const char *solver,
 		 GTK_DIALOG_DESTROY_WITH_PARENT,
 		 GTK_MESSAGE_QUESTION,
 		 GTK_BUTTONS_YES_NO,
-		 "Gnumeric is unable to locate the program <i>%s</i> needed "
-		 "for the <i>%s</i> solver.  For more information see %s.\n\n"
-		 "Would you like to locate it yourself?",
+		 _("Gnumeric is unable to locate the program <i>%s</i> needed "
+		   "for the <i>%s</i> solver.  For more information see %s.\n\n"
+		   "Would you like to locate it yourself?"),
 		 binary, solver, url);
-	title = g_strdup_printf ("Unable to locate %s", binary);
+	title = g_strdup_printf (_("Unable to locate %s"), binary);
 	g_object_set (G_OBJECT (dialog),
 		      "title", title,
 		      NULL);
@@ -1621,7 +2020,7 @@ gnm_sub_solver_locate_binary (const char *binary, const char *solver,
 		break;
 	}
 
-	title = g_strdup_printf ("Locate the %s program", binary);
+	title = g_strdup_printf (_("Locate the %s program"), binary);
 	fsel = GTK_FILE_CHOOSER
 		(g_object_new (GTK_TYPE_FILE_CHOOSER_DIALOG,
 			       "action", GTK_FILE_CHOOSER_ACTION_OPEN,
