@@ -1,7 +1,7 @@
 /* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 
 /*
- * xml-sax-write.c : export .gnumeric and the clipboard subset using a the sax
+ * xml-sax-write.c : export .gnumeric and the clipboard subset using the sax
  *			like wrappers in libgsf
  *
  * Copyright (C) 2003-2007 Jody Goldberg (jody@gnome.org)
@@ -83,9 +83,24 @@ typedef struct {
 void
 gnm_xml_out_add_gocolor (GsfXMLOut *o, char const *id, GOColor c)
 {
+	/*
+	 * This uses format "rrrr:gggg:bbbb" or "rrrr:gggg:bbbb:aaaa"
+	 * using hex numbers, i.e., the numbers are in the range from
+	 * 0 to FFFF.
+	 *
+	 * Note, that while go_xml_out_add_color exists, we cannot use
+	 * it as it using a 0-255 scaling and always includes alpha.
+	 */
 	unsigned r, g, b, a;
+	char buf[4 * 4 * sizeof (unsigned int) + 1];
+
 	GO_COLOR_TO_RGBA (c, &r, &g, &b, &a);
-	gsf_xml_out_add_color (o, id, r * 0x101, g * 0x101, b * 0x101);
+
+	sprintf (buf, "%X:%X:%X%c%X",
+		 r * 0x101, g * 0x101, b * 0x101,
+		 (a == 0xff ? 0 : ':'),
+		 a * 0x101);
+	gsf_xml_out_add_cstr_unchecked (o, id, buf);
 }
 
 static void
@@ -453,7 +468,6 @@ xml_write_style (GnmOutputXML *state, GnmStyle const *style)
 	GnmHLink   const *link;
 	GnmInputMsg const *im;
 	GnmStyleConditions const *sc;
-	GnmStyleCond const *cond;
 	GnmStyleBorderType t;
 	unsigned i;
 	gboolean started;
@@ -579,13 +593,13 @@ xml_write_style (GnmOutputXML *state, GnmStyle const *style)
 
 		parse_pos_init_sheet (&pp, (Sheet *)state->sheet);
 
-		if (v->texpr[0] != NULL &&
-		    (tmp = gnm_expr_top_as_string (v->texpr[0], &pp, state->convs)) != NULL) {
+		if (v->deps[0].texpr != NULL &&
+		    (tmp = gnm_expr_top_as_string (v->deps[0].texpr, &pp, state->convs)) != NULL) {
 			gsf_xml_out_simple_element (state->output, GNM "Expression0", tmp);
 			g_free (tmp);
 		}
-		if (v->texpr[1] != NULL &&
-		    (tmp = gnm_expr_top_as_string (v->texpr[1], &pp, state->convs)) != NULL) {
+		if (v->deps[1].texpr != NULL &&
+		    (tmp = gnm_expr_top_as_string (v->deps[1].texpr, &pp, state->convs)) != NULL) {
 			gsf_xml_out_simple_element (state->output, GNM "Expression1", tmp);
 			g_free (tmp);
 		}
@@ -605,25 +619,29 @@ xml_write_style (GnmOutputXML *state, GnmStyle const *style)
 
 	if (gnm_style_is_element_set (style, MSTYLE_CONDITIONS) &&
 	    NULL != (sc = gnm_style_get_conditions (style))) {
-		GArray const *conds = gnm_style_conditions_details (sc);
+		GPtrArray const *conds = gnm_style_conditions_details (sc);
 		if (conds != NULL) {
-			char *tmp;
 			GnmParsePos pp;
 			parse_pos_init_sheet (&pp, (Sheet *)state->sheet);
 
 			for (i = 0 ; i < conds->len ; i++) {
-				cond = &g_array_index (conds, GnmStyleCond, i);
+				unsigned ui;
+				GnmStyleCond const *cond =
+					g_ptr_array_index (conds, i);
 				gsf_xml_out_start_element (state->output, GNM "Condition");
 				gsf_xml_out_add_int (state->output, "Operator", cond->op);
-				if (cond->texpr[0] != NULL &&
-				    (tmp = gnm_expr_top_as_string (cond->texpr[0], &pp, state->convs)) != NULL) {
-					gsf_xml_out_simple_element (state->output, GNM "Expression0", tmp);
-					g_free (tmp);
-				}
-				if (cond->texpr[1] != NULL &&
-				    (tmp = gnm_expr_top_as_string (cond->texpr[1], &pp, state->convs)) != NULL) {
-					gsf_xml_out_simple_element (state->output, GNM "Expression1", tmp);
-					g_free (tmp);
+				for (ui = 0; ui < 2; ui++) {
+					GnmExprTop const *texpr = gnm_style_cond_get_expr (cond, ui);
+					char *tmp = texpr
+						? gnm_expr_top_as_string (texpr, &pp, state->convs)
+						: NULL;
+					const char *attr = (ui == 0)
+						? GNM "Expression0"
+						: GNM "Expression1";
+					if (tmp) {
+						gsf_xml_out_simple_element (state->output, attr, tmp);
+						g_free (tmp);
+					}
 				}
 				xml_write_style (state, cond->overlay);
 				gsf_xml_out_end_element (state->output); /* </Condition> */
